# tifffile/__init__.py

from .tifffile import *
from .tifffile import __all__, __doc__, __version__, main

# constants are repeated for documentation

__version__ = __version__
"""Tifffile version string."""



# tifffile/numcodecs.py

# Copyright (c) 2021-2026, Christoph Gohlke
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""TIFF codec for the Numcodecs package."""

from __future__ import annotations

__all__ = ['Tiff', 'register_codec']

from io import BytesIO
from typing import TYPE_CHECKING, Literal

from numcodecs import registry
from numcodecs.abc import Codec

from .tifffile import METADATA_DEFAULT, TiffFile, TiffWriter

if TYPE_CHECKING:
    from collections.abc import Sequence
    from typing import Any

    from .tifffile import (
        COMPRESSION,
        EXTRASAMPLE,
        PHOTOMETRIC,
        PLANARCONFIG,
        PREDICTOR,
        ByteOrder,
        TagTuple,
    )


class Tiff(Codec):  # type: ignore[misc]
    """TIFF codec for Numcodecs."""

    codec_id = 'tifffile'

    def __init__(
        self,
        # TiffFile.asarray
        key: int | slice | Sequence[int] | None = None,
        series: int | None = None,
        kind: Literal['generic', 'imagej', 'ome', 'shaped'] | None = None,
        level: int | None = None,
        squeeze: bool | None = None,
        buffersize: int | None = None,
        # TiffWriter
        bigtiff: bool = False,
        byteorder: ByteOrder | None = None,
        # TiffWriter.write
        photometric: PHOTOMETRIC | int | str | None = None,
        planarconfig: PLANARCONFIG | int | str | None = None,
        extrasamples: Sequence[EXTRASAMPLE | int | str] | None = None,
        volumetric: bool = False,
        tile: Sequence[int] | None = None,
        rowsperstrip: int | None = None,
        bitspersample: int | None = None,
        compression: COMPRESSION | int | str | None = None,
        compressionargs: dict[str, Any] | None = None,
        predictor: PREDICTOR | int | str | bool | None = None,
        subsampling: tuple[int, int] | None = None,
        metadata: dict[str, Any] | None = METADATA_DEFAULT,
        extratags: Sequence[TagTuple] | None = None,
        truncate: bool = False,
        maxworkers: int | None = None,
    ) -> None:
        self.key = key
        self.series = series
        self.kind = kind
        self.level = level
        self.squeeze = squeeze
        self.buffersize = buffersize
        self.bigtiff = bigtiff
        self.byteorder = byteorder
        self.photometric = photometric
        self.planarconfig = planarconfig
        self.extrasamples = extrasamples
        self.volumetric = volumetric
        self.tile = tile
        self.rowsperstrip = rowsperstrip
        self.bitspersample = bitspersample
        self.compression = compression
        self.compressionargs = compressionargs
        self.predictor = predictor
        self.subsampling = subsampling
        self.metadata = metadata
        self.extratags = extratags
        self.truncate = truncate
        self.maxworkers = maxworkers

    def encode(self, buf: Any) -> bytes:
        """Return TIFF file as bytes."""
        with BytesIO() as fh:
            with TiffWriter(
                fh,
                bigtiff=self.bigtiff,
                byteorder=self.byteorder,
                kind=self.kind,
            ) as tif:
                tif.write(
                    buf,
                    photometric=self.photometric,
                    planarconfig=self.planarconfig,
                    extrasamples=self.extrasamples,
                    volumetric=self.volumetric,
                    tile=self.tile,
                    rowsperstrip=self.rowsperstrip,
                    bitspersample=self.bitspersample,
                    compression=self.compression,
                    compressionargs=self.compressionargs,
                    predictor=self.predictor,
                    subsampling=self.subsampling,
                    metadata=self.metadata,
                    extratags=self.extratags,
                    truncate=self.truncate,
                    maxworkers=self.maxworkers,
                )
            return fh.getvalue()

    def decode(self, buf: Any, out: Any = None) -> Any:
        """Return decoded image as NumPy array."""
        with BytesIO(buf) as fh, TiffFile(fh) as tif:
            return tif.asarray(
                key=self.key,
                series=self.series,
                kind=self.kind,
                level=self.level,
                squeeze=self.squeeze,
                maxworkers=self.maxworkers,
                buffersize=self.buffersize,
                out=out,
            )


def register_codec(
    cls: type[Codec] = Tiff, codec_id: str | None = None
) -> None:
    """Register :py:class:`Tiff` codec with Numcodecs."""
    registry.register_codec(cls, codec_id=codec_id)



#!/usr/bin/env python3
# tifffile/lsm2bin.py

"""Convert [MP]TZCYX LSM file to series of BIN files."""

from __future__ import annotations

import argparse
import sys

try:
    from .tifffile import lsm2bin
except ImportError:
    try:
        from tifffile.tifffile import lsm2bin
    except ImportError:
        from tifffile import lsm2bin  # noqa: PLW0406


def main(argv: list[str] | None = None) -> int:
    """Lsm2bin command line usage main function."""
    parser = argparse.ArgumentParser(
        prog='lsm2bin',
        description='Convert [MP]TZCYX LSM file to series of BIN files.',
        epilog='Example: lsm2bin input.lsm output --tile 512 512',
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('lsmfile', help='path to the LSM input file')
    parser.add_argument(
        'binfile',
        nargs='?',
        help='common name of output BIN files (default: lsmfile name)',
    )
    parser.add_argument(
        '--tile',
        nargs=2,
        type=int,
        metavar=('Y', 'X'),
        help='tile Y and X dimensions (default: 256 256)',
    )
    parser.add_argument(
        '--quiet',
        action='store_true',
        help='suppress progress output',
    )
    args = parser.parse_args(None if argv is None else argv[1:])

    if args.tile is not None and any(v <= 0 for v in args.tile):
        parser.error('--tile values must be positive integers')

    tile = (args.tile[0], args.tile[1]) if args.tile is not None else None

    try:
        lsm2bin(
            args.lsmfile,
            args.binfile,
            tile=tile,
            verbose=not args.quiet,
        )
    except Exception as exc:
        print(f'{args.lsmfile}: {exc}', file=sys.stderr)
        return 1
    return 0


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3
# tifffile/tiff2fsspec.py

"""Write fsspec ReferenceFileSystem for TIFF file."""

from __future__ import annotations

import argparse
import contextlib
import json
import sys
from typing import Any

try:
    from .tifffile import tiff2fsspec
except ImportError:
    try:
        from tifffile.tifffile import tiff2fsspec
    except ImportError:
        from tifffile import tiff2fsspec  # noqa: PLW0406


def main(argv: list[str] | None = None) -> int:
    """Tiff2fsspec command line usage main function."""
    parser = argparse.ArgumentParser(
        prog='tiff2fsspec',
        description='Write fsspec ReferenceFileSystem for TIFF file.',
        epilog='Example: tiff2fsspec ./test.ome.tif https://server.com/path/',
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('tifffile', help='path to the local TIFF input file')
    parser.add_argument(
        'url', help='remote URL of TIFF file without file name'
    )
    parser.add_argument('--out', help='path to the JSON output file')
    parser.add_argument('--series', type=int, help='index of series in file')
    parser.add_argument('--level', type=int, help='index of level in series')
    parser.add_argument(
        '--key', type=int, help='index of page in file or series'
    )
    parser.add_argument(
        '--chunkmode',
        metavar='mode',
        help='mode used for chunking (int or string, e.g. "pages")',
    )
    parser.add_argument(
        '--fillvalue',
        type=float,
        help='fill value for missing data',
    )
    parser.add_argument(
        '--squeeze',
        action=argparse.BooleanOptionalAction,
        help='squeeze length-1 dimensions from zarr store',
    )
    parser.add_argument(
        '--groupname',
        help='name of the zarr group in the fsspec output',
    )
    parser.add_argument(
        '--zattrs',
        metavar='JSON',
        help='custom Zarr attributes as a JSON object string',
    )
    parser.add_argument(
        '--ref-version',
        dest='version',
        type=int,
        help='version of ReferenceFileSystem spec',
    )
    parser.add_argument(
        '--zarr-format',
        dest='zarr_format',
        type=int,
        help='Zarr format version (2 or 3)',
    )
    args = parser.parse_args(None if argv is None else argv[1:])

    chunkmode: int | str | None = args.chunkmode
    if chunkmode is not None:
        with contextlib.suppress(ValueError):
            chunkmode = int(chunkmode)

    zattrs: dict[str, Any] | None = None
    if args.zattrs is not None:
        try:
            zattrs = json.loads(args.zattrs)
        except json.JSONDecodeError as exc:
            parser.error(f'--zattrs is not valid JSON: {exc}')
        if not isinstance(zattrs, dict):
            parser.error(
                '--zattrs must be a JSON object, not an array or scalar'
            )

    try:
        tiff2fsspec(
            args.tifffile,
            args.url,
            out=args.out,
            key=args.key,
            series=args.series,
            level=args.level,
            chunkmode=chunkmode,
            fillvalue=args.fillvalue,
            squeeze=args.squeeze,
            groupname=args.groupname,
            zattrs=zattrs,
            version=args.version,
            zarr_format=args.zarr_format,
        )
    except Exception as exc:
        print(f'{args.tifffile}: {exc}', file=sys.stderr)
        return 1
    return 0


if __name__ == '__main__':
    sys.exit(main())
